/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <memory>

namespace armarx
{
    class ParseResult;
    using ParseResultPtr = std::shared_ptr<ParseResult>;

    class ParseResult
    {
    public:
        enum ErrorType { eNotSet, eNoError, eSytaxError, eContainerMismatch, eVariantParseError };
        enum ErrorHint { eNone, eOldFormat };

        ParseResult(const std::string& error, ErrorType errorType);

        bool iserr();
        std::string geterr();
        ErrorType getErrorType();
        void setErrorHint(ErrorHint errorHint);
        ErrorHint getErrorHint();

        static ParseResult Success();
        static ParseResult Error(const std::string& error, ErrorType errorType);

    private:
        std::string error;
        ErrorType errorType;
        ErrorHint errorHint;
    };
}

